/*
   Test NaN and Inf

   $Id: tNaN.c 1.5 2009/03/11 libtsp-v7r0 $
*/

#include <limits.h>
#include <math.h>
#include <stdio.h>

#include <libtsp/UTtypes.h>


int
main (int argc, const char *argv[])

{
  double dv, yv, dmant1, dmant2;
  int exp1, exp2, exp, error;

  error = 0;

  /* Check unsigned to double conversion */
  dv = (UT_uint4_t) 0xFFFFFFFF;
  yv = (UT_uint4_t) 0x7FFFFFFF;
  if (yv + 2147483648. != dv) {
    error = 1;
    fprintf (stderr, "tNaN - Error in UT_uint4_t to double conversion\n");
  }

  /* Check frexp for small values */
  dv = 4e-308;
  dmant1 = frexp (dv, &exp1);
  dmant2 = frexp (0.5*dv, &exp2);
  if (dmant1 != dmant2 && exp1 - exp2 != 1) {
    error = 1;
    fprintf (stderr, "tNaN - Error in frexp for input %g\n", 0.5*dv);
  }

  /* Check if inf - inf generates NaN */
  dv = sqrt (1.) / 0.0;		/* Inf */
  yv = -sqrt (1.) / 0.0;	/* -Inf */
  printf ("%g + %g = %g\n", dv, yv, dv + yv);

  /* Check tests with NaN */
  dv = sqrt (-1.0);		/* NaN */
  printf ("%g  < 1.: %d, %g <= 1.: %d,  %g == 1.: %d\n",
	  dv, (dv  < 1.), dv, (dv <= 1.), dv, (dv == 1.));
  printf ("%g >= 1.: %d, %g  > 1.: %d\n", dv, (dv >= 1.), dv, (dv > 1.));
  printf ("%g == %g: %d\n", dv, dv, (dv == dv));

  yv = frexp (dv, &exp);
  printf ("frexp (%g): mantissa = %g, exponent = %d\n", dv, yv, exp);

  /* Check tests with infinity */
  dv = sqrt (1.) / 0.0;		/* Inf */
  printf ("%g  < 1.: %d, %g <= 1.: %d,  %g == 1.: %d\n",
	  dv, (dv  < 1.), dv, (dv <= 1.), dv, (dv == 1.));
  printf ("%g >= 1.: %d, %g  > 1.: %d\n", dv, (dv >= 1.), dv, (dv > 1.));
  printf ("%g == %g: %d\n", dv, dv, (dv == dv));

  yv = frexp (dv, &exp);
  printf ("frexp (%g): mantissa = %g, exponent = %d\n", dv, yv, exp);

  return error;
}
